import psycopg2
from AbstractTsDbMixin import AbstractTsDbMixin
from loguru import logger
import iso8601
import datetime
from io import StringIO

class EGM1mdTsDbMixin(AbstractTsDbMixin):
    def write(self, experiment, dataObject):
        logger.info("database write operation start")
        try:
            ts = iso8601.parse_date(dataObject.payload["ts"])
            dt = datetime.timedelta(microseconds=dataObject.payload["dt"])
            values = dataObject.payload["v"]
        
            stringBuffer = ''
            cnt = 0
            for rv in values:
                unsignedValue = rv & 0x00ffffff
                valid = (rv & 0x04000000) >> 26
                sign = (rv & 0x02000000) >> 25
                signedValue = unsignedValue if not sign else unsignedValue * -1

                stringBuffer += f"{ts}\t{experiment}\t{rv}\t{valid}\t{signedValue}\n"
                ts += dt
                cnt += 1
            logger.info(f"{cnt} items")

            stringIO = StringIO(stringBuffer)

            logger.info("database copy start")
            with self.connection:
                with self.connection.cursor() as cursor:
                    cursor.copy_from(stringIO, 'egm1md', columns=('time', 'experiment', 'rawvalue', 'valid', 'value'))
            logger.info("database copy done")


        except iso8601.ParseError as e:
            logger.error(f"unable to parse timestamp, items dropped: {e}")
        except KeyError as e:
            logger.error(f"missing attribute in payload, items dropped: {e}")
        except psycopg2.Error as e:
            logger.error(f"error {e.pgcode} when talking to database, items dropped: {e}")
        
        logger.info("database write operation done")
